import numpy as np
from matplotlib import pyplot as plt

q2_interval_limits = [0.0, 0.1, 1.1, 8.0, 11.0, 12.5, 15.0, 35.0]

q2_nbins = int(len(q2_interval_limits) - 1)
q2_interval_widths = [
    q2_interval_limits[i + 1] - q2_interval_limits[i]
    for i in range(0, q2_nbins)
]
q2_interval_centers = [
    q2_interval_limits[i] + q2_interval_widths[i] / 2
    for i in range(0, q2_nbins)
]

# in units of 1e-4
R_limits_90 = [0, 1.28, 1.73, 0, 0.62, 0, 1.90]
R_limits_95 = [0, 2.14, 2.16, 0, 0.73, 0, 2.34]

for i_bin in range(0, q2_nbins):
    R_limits_90[i_bin] = R_limits_90[i_bin] * 1e-4 / q2_interval_widths[i_bin]
    R_limits_95[i_bin] = R_limits_95[i_bin] * 1e-4 / q2_interval_widths[i_bin]

total_width = 1.0 + 6.9 + 1.5 + 20.0

R_all_limit_90 = 2.06 * 1e-4 / total_width
R_all_limit_95 = 2.74 * 1e-4 / total_width

print(f"q2_interval_limits = {q2_interval_limits}")
print(f"q2_interval_widths = {q2_interval_widths}")
print(f"q2_interval_centers = {q2_interval_centers}")
print(f"R_limits_90 = {R_limits_90}")
print(f"q2_nbins = {q2_nbins}")

results = list()
for i_bin in range(0, q2_nbins):
    if R_limits_90[i_bin] == 0:
        continue
    results.append([q2_interval_centers[i_bin], R_limits_90[i_bin]])
results = np.array(results)

plt.rc("font", **{
    "family": "serif",
    "serif": "Roman",
    "stretch": "ultra-expanded"
})

plt.rc("text", usetex=True)
plt.rc("axes", linewidth=1.5)

fig = plt.figure()
ax = plt.axes([0.19, 0.2, 0.79, 0.71])

ax.plot(results[:, 0], results[:, 1], ".", color="#4040B1", markersize=0.1)

ax.set_yscale('log')

x_min = -1
x_max = 36
x_range = x_max - x_min

y_max = 5 * 1e-4
y_min = 2 * 1e-6

ax.set_xlim(-1, 36)
ax.set_ylim(y_min, y_max)

opt_95 = dict(
    color='#CCCCE6',
    lw=1.5,
    arrowstyle='->,head_width=.25,head_length=.35',
    connectionstyle='arc3,rad=0')
opt_90 = dict(
    color='#4040B1',
    lw=1.5,
    arrowstyle='->,head_width=.25,head_length=.35',
    connectionstyle='arc3,rad=0')

for i_border in range(1, len(q2_interval_limits)):
    plt.vlines(
        q2_interval_limits[i_border],
        y_min,
        y_max,
        colors='gray',
        linestyles='solid')
    if R_limits_90[i_border - 1] == 0:
        continue
    b1 = plt.axhline(
        R_limits_95[i_border - 1],
        (q2_interval_limits[i_border - 1] + 1.1) / x_range,
        (q2_interval_limits[i_border] + 0.9) / x_range,
        color="#CCCCE6",
        linestyle='solid',
        linewidth=1.5)
    plt.gca().annotate(
        '',
        xy=(q2_interval_centers[i_border - 1],
            float(0.97 * R_limits_90[i_border - 1])),
        xycoords='data',
        xytext=(q2_interval_centers[i_border - 1],
                float(1.03 * R_limits_95[i_border - 1])),
        textcoords='data',
        arrowprops=opt_95)
    b2 = plt.axhline(
        R_limits_90[i_border - 1],
        (q2_interval_limits[i_border - 1] + 1.1) / x_range,
        (q2_interval_limits[i_border] + 0.9) / x_range,
        color="#4040B1",
        linestyle='solid',
        linewidth=1.5)
    plt.gca().annotate(
        '',
        xy=(q2_interval_centers[i_border - 1],
            float(0.5 * R_limits_90[i_border - 1])),
        xycoords='data',
        xytext=(q2_interval_centers[i_border - 1],
                float(1.03 * R_limits_90[i_border - 1])),
        textcoords='data',
        arrowprops=opt_90)

plt.axhline(
    R_all_limit_95, (0.1 + 1.1) / x_range, (35 + 0.9) / x_range,
    color="#CCCCE6",
    linestyle='dashed',
    linewidth=1.5,
    zorder=0)

plt.axhline(
    R_all_limit_90, (0.1 + 1.1) / x_range, (35 + 0.9) / x_range,
    color="#4040B1",
    linestyle='dashed',
    linewidth=1.5,
    zorder=0)

ax.axvspan(8.0, 11.0, color='grey', alpha=1.0)
ax.axvspan(12.5, 15.0, color='grey', alpha=1.0)

ax.set_ylabel(
    r"$\frac{{\rm d}\mathcal{B}(B_{c}^{+}\to\pi^{+}\mu^{+}\mu^{-})/{\rm d}q^{2}}{\mathcal{B}(B_{c}^{+}\to J/\psi\pi^{+})}\,[{\rm GeV}^{-2}]$",
    fontsize=22,
    labelpad=10)
ax.set_xlabel(r"$q^2\,[{\rm GeV}^{2}]$", fontsize=22)

x_tickmarks = [0, 5, 10, 15, 20, 25, 30, 35]  # q2_interval_limits[1:]
x_tickmarks_labels = ["{:.1f}".format(i_tick) for i_tick in x_tickmarks]

plt.xticks(x_tickmarks, x_tickmarks_labels)

ax.tick_params(
    axis='both',
    which='major',
    top=True,
    right=True,
    labelsize=20,
    direction="in",
    length=10,
    width=1.5)
ax.tick_params(
    axis='y',
    which='minor',
    top=True,
    right=True,
    direction="in",
    length=5,
    width=1.5)

plt.text(26.5, 1e-4, "LHCb", fontsize=25, fontweight="bold")
plt.text(26.5, 5.5e-5, r"$9\,{\rm fb}^{-1}$", fontsize=20)
plt.text(8.8, 1e-4, r"$J/\psi$", fontsize=15, rotation=90)
plt.text(13.0, 1e-4, r"$\psi(2S)$", fontsize=15, rotation=90)

ax.legend([b1, b2], [r"$95\%\ {\rm CL}$", r"$90\%\ {\rm CL}$"],
          fontsize=13,
          loc='center right',
          frameon=False)

plt.savefig("Fig6.pdf")

print("Created: Fig6.pdf")
